@@ -4,7 +4,7 @@
  *
  * pgpool-recovery: exec online recovery script from SELECT statement.
  *
- * Copyright (c) 2003-2013	PgPool Global Development Group
+ * Copyright (c) 2003-2018	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -83,13 +83,13 @@ pgpool_recovery(PG_FUNCTION_ARGS)
 	{
 		char *remote_port = DatumGetCString(DirectFunctionCall1(textout,
 														  PointerGetDatum(PG_GETARG_TEXT_P(3))));
-		snprintf(recovery_script, sizeof(recovery_script), "%s/%s %s %s %s %s",
+		snprintf(recovery_script, sizeof(recovery_script), "\"%s/%s\" \"%s\" \"%s\" \"%s\" \"%s\"",
 				 DataDir, script, DataDir, remote_host,
 				 remote_data_directory, remote_port);
 	}
 	else
 	{
-		snprintf(recovery_script, sizeof(recovery_script), "%s/%s %s %s %s",
+		snprintf(recovery_script, sizeof(recovery_script), "\"%s/%s\" \"%s\" \"%s\" \"%s\"",
 				 DataDir, script, DataDir, remote_host,
 				 remote_data_directory);
 	}
@@ -233,7 +233,14 @@ pgpool_switch_xlog(PG_FUNCTION_ARGS)
 	xlogfile_name_oid = get_function_oid("pg_xlogfile_name", pg_xlogfile_name_arg_type, "pg_catalog");
 
 	if (!switch_xlog_oid || !xlogfile_name_oid)
-		elog(ERROR, "cannot find xlog functions");
+	{
+		/* probably PostgreSQL is 10 or greater */
+		switch_xlog_oid = get_function_oid("pg_switch_wal", NULL, "pg_catalog");
+		xlogfile_name_oid = get_function_oid("pg_walfile_name", pg_xlogfile_name_arg_type, "pg_catalog");
+
+		if (!switch_xlog_oid || !xlogfile_name_oid)
+			elog(ERROR, "cannot find xlog functions");
+	}
 
 	location = OidFunctionCall1(switch_xlog_oid, PointerGetDatum(NULL));
 	filename_t = DatumGetTextP(OidFunctionCall1(xlogfile_name_oid, location));
